// aromaticity.c
//
// 17 August 2004 K.Tonomura
//

#include "aromaticity.h"

int double_bond(int member, int **cmat, int dim) {
  int res = 0;
  for( int i=0 ; i < dim ; i ++ ) {
    if( cmat[member][i] == 2 ) {
      res ++;
    }
  }
  return res;
}

int hetero_atom(int member, int **cmat, molform moltmp) {

  int res = 0;
  int charge = 0;
  if( charge > 0 && charge < 8 ) {
    charge = 4 - moltmp.charge[member];
  }

  if( moltmp.atomsymbol[member] == "N" ) {
    for( int i=0 ; i < moltmp.nodenum ; i ++ ) {
      
      if( cmat[member][i] > 1 ) {
	return 0; // 片側に二重結合があれば、 N の .Fpp電子は二重結合の数だけ
      }
      if( cmat[member][i] == 1 )
	res ++;
    }

    //    cout << "hetero N:" << res << endl;
    
    if( res == 2 || res == 3 )
      return 1;
    else
      return 0;

  } else if (  moltmp.atomsymbol[member] == "O" ) {
    for( int i=0 ; i < moltmp.nodenum ; i ++ ) {
      if( cmat[member][i] > 1 ) {
	return 0;
      }
      if( cmat[member][i] == 1 )
	res ++;
    }
    //        cout << "hetero O:" << res << endl;

    if( res == 2 || res == 3 ) // ま、ring なんだし、普通は 2 の筈だよね -O- 
      return 1; // 環内Pi結合を考慮して 2 のところ1にしてる?
    else
      return 0;
    
  } else if (  moltmp.atomsymbol[member] == "S" ) {
    for( int i=0 ; i < moltmp.nodenum ; i ++ ) {
      if( cmat[member][i] > 1 ) {
	return 0;
      }
      if( cmat[member][i] == 1 )
	res ++;
    }
    if( res == 2 || res == 3 ) // O-like と考えて、同じ処理
      return 1; // 環内Pi結合を考慮して 2 のところ1にしてる?
    else
      return 0;
  }
  return 0;
}

// 奥野さんのperlを元に算出したhukel則のサブルーチン

int huckel_check(molform moltmp , int **cmat , int *set , int **cpair  , int r_flg ) {

  // r_flg == 0 ; node , r_fld == 1 ; edge
 
  int huckel = 0;
  int nonsp2 = 0;
  int *ring;
  int metal = 0;

  int outline;

  if( r_flg == 0 ) {
    ring = index_copy( set , moltmp.nodenum + 1 );
    outline = set[0]; 
  } else {

    ring = index_new(moltmp.nodenum + 1);
    ring[0] = 0;
  
    for( int i = 1 ; i <= set[0] ; i ++ ) {

      int j = set[i];
   
      int *current_pair;
      current_pair = index_new(3);
      current_pair[0] = 2;
      current_pair[1] = cpair[ j ][0];
      current_pair[2] = cpair[ j ][1];
	
      add_node( current_pair , ring );
      index_del( current_pair );

    }
    outline = set[ set[0] + 1 ];
  }

  for( int i = 1 ; i <= ring[0] ; i++ ) {

    if( checkmetal(moltmp.atomsymbol[ ring[ i ] ]) == 1 )
      metal = 1;

    //    if( hetero_atom(ring[i],cmat,moltmp) )
    //      hukel ++;


    huckel = huckel + hetero_atom(ring[i],cmat,moltmp);
    
    if( moltmp.atomsymbol[ring[i]] == "C" && double_bond(ring[i],cmat,moltmp.nodenum) == 0 )
      nonsp2 = 1;
  }

  if( metal == 1)
    return 0;

  for( int i = 1 ; i <= ring[0] ; i++ ) {
    for( int j = i ; j <= ring[0] ; j ++ ) {
      if( ring[i] != ring[j] ) {
	if( cmat[ring[i]][ring[j]] == 2 ) {
	  huckel ++;
	}
      }
    }
  }
  // cout << "huckel = " << hukel << ": non_sp2= " << nonsp2 << ": outline = " << outline << endl;

  index_del( ring );
  
  if( huckel * 2 >= outline && huckel%2 == 1 && nonsp2 != 1 )
    return 1;
  else
    return 0;
}
