// cpdformat.c
//
// 7 Decembre 2004 K.Tonomura
//

#include "cpdformat.h"

int parity = 0;  // set parity = 1 when you want to write parity.

molform read_Molfile(FILE *fp , bool ihflg)
{
  char buf[256];
  char *bufs;
  int i = 0;
  int j = 0;
  int j_wk = 0;
  int iH = 0;
  int iE = 0;
  int *ieH;
  int inode = 0 , ibond = 0 ;
  molform moltmp;

  while( fgets(buf , 256 , fp ) != NULL) {
    buf[strlen(buf) - 1 ] = '\0';
    if( buf[0] !=  '\0' ) {
      i ++;

      if( strstr( buf , "V2000" ) != NULL && i < 4 ) {
        i = 4; 
      }

      if( i == 4 ) {
	inode = getint(buf,0,2);
	//	moltmp.nodenum = getint(buf,0,2);
	ibond = getint(buf,3,5);

	//	cout << inode << ":" << ibond << endl;
	moltmp.chiral =  getint(buf,12,14);

	ieH = index_new(inode + 1);
	for( int k = 0 ; k <= inode ; k ++ ) 
	  ieH[k] = k;

      } else if( i > 4 ) {
	if( inode > 0 && j < inode ) {

	  int iF = 1;
	  if( ! ihflg ) {
	    bufs = getstr_line(buf,31,33);
	    if( del_space(string(bufs)) == "H" ) {
	      cout << "ignore explicit Hydrogen. (" << j + 1 << ")" << endl; 
	      iF = 0;
	      iH ++; 
	      ieH[j] = -1;
	    } else {
	      ieH[j] = j - iH;
	    }
	  }

	  j_wk = j - iH;

	  if( iF == 1 ) {

	    moltmp.x[j_wk] = getdouble(buf,0,10);
	    moltmp.y[j_wk] = getdouble(buf,11,20);
	    moltmp.z[j_wk] = getdouble(buf,21,30);
	    bufs = getstr_line(buf,31,33);
	    moltmp.atomsymbol[j_wk] = del_space(string(bufs));
	  
	    moltmp.massdifference[j_wk] = getint(buf,34,35);
	    moltmp.charge[j_wk] =  getint(buf,36,38);
	    moltmp.stereoparity[j_wk] =  getint(buf,39,41);
	    moltmp.hydrogencount[j_wk] = getint(buf,42,44);
	    moltmp.stereocare[j_wk] = getint(buf,45,47);
	    moltmp.valence[j_wk] = getint(buf,48,50);
	  }
	  
	  j ++;

	} else if ( j >= inode && j < inode + ibond){
	  
	  int istart =  getint(buf,0,2) - 1;
	  int iend = getint(buf,3,5) - 1;

	  int iF = 1;
	  if( ! ihflg ) {
	    if( ieH[ istart ] == -1 || ieH[ iend ] == -1 ) {
	      cout << "ignore explicit Hydrogen edge. (" << istart + 1 << "-" << iend + 1 << ")" << endl; 

	      iF = 0;
	      iE ++;
	    }
	  }
	  
	  if( iF == 1 ) {
	    moltmp.bond.s[j - inode - iE] = ieH[ getint(buf,0,2) - 1 ] + 1;
	    moltmp.bond.e[j - inode - iE] = ieH[ getint(buf,3,5) - 1 ] + 1;
	    moltmp.bond.bondtype[j - inode - iE] = getint(buf,6,8);
	    moltmp.bond.stereo[j - inode - iE] = getint(buf,9,11);
	  }
	  j ++;

	}
      }
    }
  }
  
  moltmp.nodenum = inode - iH;
  moltmp.bondnum = ibond - iE;

  moltmp.index[0] = inode;
  for( i = 0 ; i < inode ; i ++ ) {
    moltmp.index[i + 1] = ieH[i] + 1;
  }

  index_del(ieH);
  return (moltmp);
 
}

brackets read_Bracket(FILE *fp)
{
  char buf[256];
  int imember = 0;

  brackets brs;
  brs.member = 0;

  while( fgets(buf , 256 , fp ) != NULL) {
    buf[strlen(buf) - 1 ] = '\0';
    if( buf[0] != '\0' ) {
      if( buf[0] == 'M' ) {

	string swk = "STY";
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,8);
	 
	  if (imember > 0 ) {
	    brs.member = imember;
	    char *btmp = trim(getstr_line(buf,9, strlen(buf) - 1));

	    for( int i = 0 ; i < imember && i < 20 ; i ++ ) {
	      brs.br[i].count = i + 1;
	      brs.br[i].type = TokenizeStringSp(btmp, i * 2 + 1  ); 
	      brs.br[i].uopt = "";

	      brs.br[i].l1.x1 = 0.0;
	      brs.br[i].l1.y1 = 0.0;
	      brs.br[i].l1.x2 = 0.0;
	      brs.br[i].l1.y2 = 0.0;
	      brs.br[i].l1.set = false;
	      brs.br[i].uopt = ""; 
	      brs.br[i].l2.x1 = 0.0;
	      brs.br[i].l2.y1 = 0.0;
	      brs.br[i].l2.x2 = 0.0;
	      brs.br[i].l2.y2 = 0.0;
	      brs.br[i].l2.set = false;
	      brs.br[i].dopt = "";

	      brs.br[i].rep_atom_num = 0;
	      brs.br[i].org_atom_num = 0;

	      if( brs.br[i].type == "SRU" ) 
		brs.br[i].dopt = "n";
	      else if( brs.br[i].type == "GEN" ) 
		brs.br[i].dopt = "1";
	      else
		brs.br[i].dopt = brs.br[i].type;

	    }
	  }
	  
	}
	swk = "SCN"; // 右肩のところの添字だよ
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,8);
	  if (imember > 0 ) {
	    char *btmp = trim(getstr_line(buf,9, strlen(buf) - 1));
	    for( int i = 0 ; i < imember && i < 20 ; i ++ ) {
	      int iwk = atoi( TokenizeStringSp(btmp, i * 2 ) );
	      if( iwk <= imember) 
		brs.br[iwk - 1].uopt = TokenizeStringSp(btmp, i * 2 + 1  ); 
	   
	    }
	  }
	}

	swk = "SMT"; // 右下のところの添字だよ SST は無視する。
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,9);
	  if (imember > 0 ) {
	    char *btmp = trim(getstr_line(buf,6 , strlen(buf) - 1));
	    int iwk = atoi( TokenizeStringSp(btmp, 0 ) );
	    if( iwk <= imember ) 
	      brs.br[iwk - 1].dopt = TokenizeStringSp(btmp,1  ); 
	  }
	}

	swk = "SAL"; // 括弧の中の原子のリスト REPEAT
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,9);
	  if (imember > 0 ) {
	    char *btmp = trim(getstr_line(buf,6 , strlen(buf) - 1));
	    int iwk = atoi( TokenizeStringSp(btmp, 0 ) );
	    int jwk = atoi( TokenizeStringSp(btmp, 1 ) );

	    for(int i = 0; i < jwk; i++){
	      brs.br[iwk - 1].rep_atom[ brs.br[iwk - 1].rep_atom_num++ ]
		= atoi( TokenizeStringSp(btmp, i+2));
	    }

	  }
	}

	swk = "SPA"; // 括弧の中の原子のリスト ORIGINAL
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,9);
	  if (imember > 0 ) {
	    char *btmp = trim(getstr_line(buf,6 , strlen(buf) - 1));
	    int iwk = atoi( TokenizeStringSp(btmp, 0 ) );
	    int jwk = atoi( TokenizeStringSp(btmp, 1 ) );

	    for(int i = 0; i < jwk; i++){
	      brs.br[iwk - 1].org_atom[ brs.br[iwk - 1].org_atom_num++ ]
		= atoi( TokenizeStringSp(btmp, i+2));
	    }

	  }
	}

	swk = "SDI"; // 座標が入ります
	if( getstr_line(buf,3,5) == swk  ) {
	  imember = getint(buf,6,9);
	  if (imember > 0 ) {
	    char *btmp = trim(getstr_line(buf,14, strlen(buf) - 1));
	    double x1,y1,x2,y2;
	    x1 = atof( TokenizeStringSp(btmp, 0  ) );
	    y1 = atof( TokenizeStringSp(btmp, 1  ) );
	    x2 = atof( TokenizeStringSp(btmp, 2  ) );
	    y2 = atof( TokenizeStringSp(btmp, 3  ) );
	  
	    //	    cout << "braket : (" << x1 << "," << y1 << ")-(" << x2 << "," << y2 << ")" << endl; 
	    if( brs.br[imember - 1].l1.set == false ) {
	      brs.br[imember - 1].l1.x1 = x1;
	      brs.br[imember - 1].l1.y1 = y1;
	      brs.br[imember - 1].l1.x2 = x2;
	      brs.br[imember - 1].l1.y2 = y2;
	      brs.br[imember - 1].l1.set = true;

	    } else {
	      brs.br[imember - 1].l2.x1 = x1;
	      brs.br[imember - 1].l2.y1 = y1;
	      brs.br[imember - 1].l2.x2 = x2;
	      brs.br[imember - 1].l2.y2 = y2;
	      brs.br[imember - 1].l2.set = true;

	    }
	  }
	}
      }
    }
  }
  return brs;
}

molform read_Kcffile(FILE *fp , bool ihflg)
{
  char buf[256];
  int i = 0;
  int j = 0;
  int j_wk = 0;
  int iH = 0;
  int iE = 0;
  int *ieH;
  int inode = 0, ibond = 0;
  molform moltmp;

  int line_status = 0;

  while( fgets(buf , 256 , fp ) != NULL) {
    buf[strlen(buf) - 1 ] = '\0';
    if( buf[0] != '\0' ){
      i ++;

      if( line_status == 0  ) {
	if( strstr( getstr_line(buf,0,4) , "ENTRY" ) != NULL && strstr( getstr_line(buf,35,50) , "Compound" ) != NULL ) {
	  //	  cout << " kcf!!! " << endl;

	  //	  if( strstr( buf , "#chiral" ) ) {
	  //	    moltmp.chiral = 1;
	  //	  } else {
	  //	    moltmp.chiral = 0;
	  //	  }
	  line_status = 1;
	}
      } else if( line_status == 1 ) {

	if( strstr( getstr_line(buf,0,3) , "ATOM" ) != NULL || strstr( getstr_line(buf,0,3) , "NODE" ) != NULL ) {
	  inode = getint(buf,11,14);
	  line_status = 2;
	  j = 0;

	  ieH = index_new(inode + 1);
	  for( int k = 0 ; k <= inode ; k ++ ) 
	    ieH[k] = k;
	} 
      } else if( line_status == 2 ) {

	if( strstr( getstr_line(buf,0,3) , "BOND" ) != NULL || strstr( getstr_line(buf,0,3) , "EDGE" ) != NULL ) {

	  ibond = getint(buf,11,14);
	  //	  cout << "ibond=" << ibond << endl;
	  if( ibond == 0 ) { 
	    line_status = 4;
	    moltmp.bondnum = 0;
	  } else 
	    line_status = 3;
	  j = 0;

	} else {
 
	  // ATOM line
	  char *token;
	  int col = 0;

	  int iwk;
	  string wkcf;
	  string watom;
	  double wx,wy,wz;
	  int wc = 0;

	  char cbuf[256];
	  strcpy(cbuf , buf);
	  token = strtok(cbuf," ");

	  while ( token != NULL) {
	    col ++;

	    if( col == 1 ) {
	      iwk = atoi(token);
	    } else if( col == 2 ) {
	      wkcf = token;
	    } else if( col == 3 ) {
	      watom = token;
	    } else if( col == 4 ) {
	      if( check_num(token) == 0 ) 
		wx = atof(token);
	      else 
		wx = 0.0;
	    } else if( col == 5 ) {
	      if( check_num(token) == 0 ) 
		wy = atof(token);
	      else 
		wy = 0.0;
	    } else if( col > 5 ) {
	      if( check_num(token) == 0 ) {
		if( col == 6 )
		  wz = atof(token);
		else 
		  wz = 0.0;
	      }
	    }
	  
	    token = strtok(NULL," ");
	    if (token==NULL)
	      break;
	  }

	  if( strstr( buf , "#3+") != NULL ) {
	    wc = 1;
	  } else if( strstr( buf , "#2+") != NULL ) {
	    wc = 2;
	  } else if( strstr( buf , "#+") != NULL ) {
	    wc = 3;
	  } else if( strstr( buf , "#^") != NULL ) {
	    wc = 4;
	  } else if( strstr( buf , "#-") != NULL ) {
	    wc = 5;
	  } else if( strstr( buf , "#2-") != NULL ) {
	    wc = 6;
	  } else if( strstr( buf , "#3-") != NULL ) {
	    wc = 7;
	  }

	  int iF = 1;
	  if( ! ihflg ) {
	    if( del_space(watom) == "H" ) {
	      cout << "ignore explicit Hydrogen. (" << j + 1 << ")" << endl; 
	      iF = 0;
	      iH ++; 
	      ieH[j] = -1;
	    } else {
	      ieH[j] = j - iH;
	    }
	  }
	  j_wk = j - iH;

	  if( iF == 1 ) {

	    moltmp.x[j_wk] = wx;
	    moltmp.y[j_wk] = wy;
	    moltmp.z[j_wk] = wz;
	    moltmp.atomsymbol[j_wk] = watom;
	    moltmp.kcfsymbol[j_wk] = wkcf;

	    moltmp.massdifference[j_wk] = 0;
	    moltmp.charge[j_wk] =  wc;
	    moltmp.stereoparity[j_wk] =  0;
	    moltmp.hydrogencount[j_wk] = 0;
	    moltmp.stereocare[j_wk] = 0;
	    moltmp.valence[j_wk] = 0;

	  }	  
	  j ++;
	}

      } else if( line_status == 3 ) {

	char *token;
	int col = 0;

	int iwk,isw,iew,ib;
	int bs = 0;

	char cbuf[256];
	strcpy(cbuf , buf);
	token = strtok(cbuf," ");

	while ( token != NULL) {
	  col ++;
 
	  if( col == 1 ) {
	    iwk = atoi(token);
	  } else if( col == 2 ) {
	    isw = atoi(token);
	  } else if( col == 3 ) {
	    iew = atoi(token);
	  } else if( col == 4 ) {
	    ib = atoi(token);
	  } else if( col > 4 ) {
	      // #Down とか #Up とか どうしよう?		
	  }

	  if( strstr( buf , "#Up") != NULL ) {
	    bs = 1;
	  } else if( strstr( buf , "#Either") != NULL ) {
	    bs = 4;
	  } else if( strstr( buf , "#Down") != NULL ) {
	    bs = 6;
	  }

	  token = strtok(NULL," ");
	  if (token==NULL)
	    break;
	}
	int istart =  isw - 1;
	int iend = iew - 1;

	int iF = 1;
	if( ! ihflg ) {
	  if( ieH[ istart ] == -1 || ieH[ iend ] == -1 ) {
	    cout << "ignore explicit Hydrogen edge. (" << istart + 1 << "-" << iend + 1 << ")" << endl; 
	    
	    iF = 0;
	    iE ++;
	  }
	}
	  
	if( iF == 1 ) {
	  moltmp.bond.s[j - iE] = ieH[ istart ] + 1;
	  moltmp.bond.e[j - iE] = ieH[ iend ] + 1;
	  moltmp.bond.bondtype[j - iE] = ib;
	  moltmp.bond.stereo[j - iE] = bs;
	}
	j ++;
      }    
    }
  }
  
  moltmp.nodenum = inode - iH;
  moltmp.bondnum = ibond - iE;

  moltmp.index[0] = inode;
  for( i = 0 ; i < inode ; i ++ ) {
    //    cout << i << "->" <<  ieH[i] << " " << endl;
    moltmp.index[i + 1] = ieH[i] + 1;
  }

  index_del(ieH);
  //  cout << "douyo" << endl;
  return (moltmp);
 
}

void output_kcffile(molform moltmp , brackets bra ,char *sFile , bool b3D , bool acolor )
{

  ofstream fout(sFile); 
  char buffer[200];

  sprintf(buffer , "ENTRY       %-27s Compound",moltmp.name);
  fout << buffer;
  fout << endl;

  sprintf(buffer,"ATOM        %d\n",moltmp.nodenum);

  fout << buffer;
  if( moltmp.nodenum > 0) {
    for(int i=0; i< moltmp.nodenum ; i++ ){
      fout << "            "; // 12spaces
      fout << i + 1 << " ";

      if( i < 9 )
	fout << "  ";
      else if( i < 99 ) 
	fout << " ";

      int ilen = moltmp.kcfsymbol[i].length();
      fout << moltmp.kcfsymbol[i];

      if( ilen < 3 ) 
	for( int j=ilen; j<3; ++j)
	  fout << " ";
      
      fout << " " << moltmp.atomsymbol[i];
      if(moltmp.atomsymbol[i].length() == 1 )
	fout << " ";
      
      if( b3D ) {
	sprintf(buffer," %9.4f %9.4f %9.4f",moltmp.x[i],moltmp.y[i],moltmp.z[i]);
      } else {
	sprintf(buffer," %9.4f %9.4f",moltmp.x[i],moltmp.y[i]);
      }
      fout << buffer;
      
      if( moltmp.charge[i] == 1 ) 
	fout << " #3+";
      if( moltmp.charge[i] == 2 )
	fout << " #2+";
      if( moltmp.charge[i] == 3 )
	fout << " #+";
      if( moltmp.charge[i] == 4 )
	fout << " #^";
      if( moltmp.charge[i] == 5 ) 
	fout << " #-";
      if( moltmp.charge[i] == 6 )
	fout << " #2-";
      if( moltmp.charge[i] == 7 )
	fout << " #3-";

      if( parity == 1 ) {
	if( moltmp.stereoparity[i] == 1 )
	  fout << " #R";
	else if( moltmp.stereoparity[i] == 2 )
	  fout << " #S";
	else if( moltmp.stereoparity[i] == 3 )
	  fout << " #Either";
      } else {
	//	if( moltmp.stereoparity[i] == 3 )
	//	  fout << " #Either";
      }
      fout << endl;
    }
  }
  
  sprintf(buffer,"BOND        %d\n",moltmp.bondnum);

  fout << buffer;
  
  if( moltmp.bondnum > 0) {
    
    for(int i=0; i< moltmp.bondnum ; i++ ){
      //      cout << "123456789+123456789+123456789+123456789" << endl;
      
      fout << "            ";
      fout << i + 1 << " ";

      if( i < 9 )
	fout << "  ";
      else if( i < 99 ) 
	fout << " ";

      sprintf(buffer,"%3d %3d %d",moltmp.bond.s[i],moltmp.bond.e[i],moltmp.bond.bondtype[i] );
      fout << buffer;

      if( moltmp.bond.stereo[i] == 1 )
	fout << " #Up";
      else if( moltmp.bond.stereo[i] == 4 )
	fout << " #Either";
      else if( moltmp.bond.stereo[i] == 6 )
	fout << " #Down";

      if( acolor && ( moltmp.bond.ring[i] == 2 || moltmp.bond.bondtype[i] == 4 ) ) 
      	fout << " #00FFFF";

      fout << endl;
      
    }
  }

  if( bra.member > 0 ) {
    //    cout << "(?_?)" << endl; 
    string strLabel = "BRACKET     ";
    int ibra = 0;
    for( int i= 0 ; i < bra.member ; i ++ ) {

      // 座標がセットされていない場合は、braketと認めない
      // 両側の座標が必要なものとする b
      if( bra.br[i].l1.set == true && bra.br[i].l2.set == true ) {
	ibra ++ ;
	fout << strLabel;
	sprintf(buffer,"%-2d %9.4f %9.4f %9.4f %9.4f\n",ibra,bra.br[i].l1.x1,bra.br[i].l1.y1,bra.br[i].l1.x2,bra.br[i].l1.y2);
	fout << buffer;
	strLabel = "            ";
	fout << strLabel;
	sprintf(buffer,"%-2d %9.4f %9.4f %9.4f %9.4f\n",ibra,bra.br[i].l2.x1,bra.br[i].l2.y1,bra.br[i].l2.x2,bra.br[i].l2.y2);
	fout << buffer;
	if( bra.br[i].uopt != "" && bra.br[i].uopt != "HT" ) {
	  fout << strLabel;
	  sprintf(buffer,"%-2d ",ibra);
	  fout << buffer;
	  fout << bra.br[i].uopt;
	  fout << endl;
	}
	if( bra.br[i].dopt != "" ) {
	  fout << strLabel;
	  sprintf(buffer,"%-2d ",ibra);
	  fout << buffer;
	  fout << bra.br[i].dopt;
	  if( bra.br[i].type == "GEN" ) {
	    sprintf(buffer," #GEN");
	    fout << buffer;
	  }
	  fout << endl;
	}
	// ORIGINAL, REPEAT の出力
	if(bra.br[i].org_atom_num == 0 && bra.br[i].rep_atom_num != 0){
	  for(int j = 0; j < bra.br[i].rep_atom_num; j++)
	    bra.br[i].org_atom[j] = bra.br[i].rep_atom[j];
	  bra.br[i].org_atom_num = bra.br[i].rep_atom_num;
	}

	if(bra.br[i].org_atom_num != 0 &&
	   bra.br[i].rep_atom_num >= bra.br[i].org_atom_num &&
	   bra.br[i].type != "GEN"){

	  // ORIGINAL
	  sprintf(buffer,"  ORIGINAL  %-2d",ibra);
	  fout << buffer;
	  int n = -1;
	  for(int j = 0; j < bra.br[i].org_atom_num; j++){
	    n++;
	    if(n != 0 && (n % 16 == 0)){
	      fout << endl;
	      sprintf(buffer,"            %-2d",ibra);
	      fout << buffer;
	    }
	    sprintf(buffer," %3d", bra.br[i].org_atom[j]);
	    fout << buffer;
	  }
	  fout << endl;

	  // REPEAT
	  sprintf(buffer,"  REPEAT    %-2d",ibra);
	  fout << buffer;
	  n = -1;
	  for(int j = 0; j < bra.br[i].rep_atom_num; j++){
	    int k;
	    for(k = 0; k < bra.br[i].org_atom_num; k++){
	      if(bra.br[i].rep_atom[j] == bra.br[i].org_atom[k]) break;
	    }
	    if(k == bra.br[i].org_atom_num){
	      n++;
	      if(n != 0 && (n % 16 == 0)){
		fout << endl;
		sprintf(buffer,"            %-2d",ibra);
		fout << buffer;
	      }
	      sprintf(buffer," %3d", bra.br[i].rep_atom[j]);
	      fout << buffer;
	    }
	  }
	  fout << endl;
	}

      }
    }
  }

  fout << "///" << endl;
  fout.close();

}
 
void output_Molfile(molform moltmp , char *sFile )
{

  ofstream fout(sFile); 

  char buffer[200];
  //  fout << "<START>" << endl;
  fout << endl;
  //  cout << "  ISISHOST03240423012D 1   1.00000     0.00000    12" << endl;
  fout << "  EXTRACT-MOL FILES " << endl;
  fout << endl;

  sprintf(buffer , "%3d%3d%3d%3d%3d%3d            999 V2000\n",moltmp.nodenum,moltmp.bondnum,0,0,moltmp.chiral,0); 
  fout << buffer;
  //          aaabbblllfffcccsssxxxrrrpppiiimmmvvvvvv
  if( moltmp.nodenum > 0) {
    for(int i=0; i< moltmp.nodenum ; i++ ){

      sprintf(buffer,"%10.4f%10.4f%10.4f ",moltmp.x[i],moltmp.y[i],moltmp.z[i]);

      fout << buffer;
      fout << moltmp.atomsymbol[i];
      
      if( moltmp.atomsymbol[i].length() == 2 ) 
      	fout << " ";
      else if( moltmp.atomsymbol[i].length() == 1 )
	fout << "  ";

      sprintf(buffer,"%2d%3d%3d%3d%3d%3d",moltmp.massdifference[i],moltmp.charge[i],moltmp.stereoparity[i],moltmp.hydrogencount[i],moltmp.stereocare[i],0);
      fout << buffer;
      fout << "           0  0  0";

      fout << endl;
    }
    if( moltmp.bondnum > 0) {
      for(int i=0; i< moltmp.bondnum ; i++ ){

	sprintf(buffer,"%3d%3d%3d%3d   %3d%3d",moltmp.bond.s[i],moltmp.bond.e[i],moltmp.bond.bondtype[i],moltmp.bond.stereo[i],0,0);
	fout << buffer << endl;
      }
    }
  }  
  fout << "M  END" << endl;
  fout.close();
}

void output_mol_aromaticity(char *sinFile, char *soutFile, int *ring_flg)
{

  FILE *fp;
  fp = fopen(sinFile,"r");

  ofstream fout(soutFile);
  bool oflg;
  char buf[256];
  char wbuf[256];

  int i = 0;
  int j = 0;
  int nodenum , bondnum;
  int intAroma_s,intAroma_e,intBondAroma;

  while( fgets(buf , 256 , fp)  ) {
    buf[strlen(buf) - 1 ] = '\0';
    oflg = true;

    //    cout << '"' << buf << '"' << endl;
    if( buf[0] != '\0' ) {
      i ++;
      if( i == 4 ) {
	nodenum = getint(buf,0,2);
	bondnum = getint(buf,3,5);
	
      } else if( i > 4 ) {
	if( nodenum > 0 && j < nodenum) {
	  j ++;
	} else if ( j >= nodenum && j < nodenum + bondnum){
	  intAroma_s = getint(buf,0,2) - 1;
	  intAroma_e = getint(buf,3,5) - 1;
	  intBondAroma = getint(buf,6,8);

	  if( ring_flg[intAroma_s] == 2 && ring_flg[intAroma_e] == 2 ) {
	    //	    if( intBondAroma < 2 ) {
	      intBondAroma = 4;
	      //	    }
            fout << getstr_line(buf,0,5);
	    sprintf(wbuf,"%3d",intBondAroma);
	    fout << wbuf;
	    fout << getstr_line(buf,9,strlen(buf));
	    fout << endl;
	    oflg = false;
	  } 
	  j ++;
	}
      }
    }
    if( oflg ) {
      fout << buf << endl;;
    }
  }
  fclose(fp);
  fout.close();
  
}

bool check_aromabond(molform moltmp) {
  bool ret = false;
  for(int i=0; i< moltmp.bondnum ; i++ ){
    //  printf("%d %d\n",j,k);
    if( moltmp.bond.bondtype[i] == 4 ) {
      ret = true;
    }
  }
  return (ret);
}

int check_aromanode(molform moltmp , int node) {
  int ret = 0;
// modify 20091027 start 
    int wnode = node + 1;
  //int wnode = node;
// modify end
  for(int i=0; i< moltmp.bondnum ; i++ ){

    if( moltmp.bond.s[i] == wnode || moltmp.bond.e[i] == wnode ) {
      if( moltmp.bond.bondtype[i] == 4 ) {
        ret ++;
      }
    }
  }
  return (ret);
}

/*
void  set_cmatrix(int **a, molform moltmp){
  if( moltmp.bondnum > 0) {
    for(int i=0; i< moltmp.bondnum ; i++ ){
      //  printf("%d %d\n",j,k);
      a[moltmp.bond.s[i] - 1 ][moltmp.bond.e[i] - 1 ] = 1;
      a[moltmp.bond.e[i] - 1 ][moltmp.bond.s[i] - 1 ] = 1;
    }
  } 
}
*/
