// makekcf.cpp
//
// version 1.1.0 / 16 January 2009 Tomohiro OHYA
// version 1.0.0 / 25 December 2008 Tomohiro OHYA
// version 0.2.6 / 15 August 2006 Masahiro HATTORI
// version 0.2.5 / 28 July 2006 Masahiro HATTORI
//
// 2 june 2005 Koichiro Tonomura
// version0.2.4b
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <iostream>
#include <time.h>

#include "t_string.h"
#include "t_matrix.h"
#include "cpdformat.h"
#include "t_graphtool.h"
#include "search_rings.h"
#include "classify_atom.h"
#include "aromaticity.h"
#include "kcfvector.h"

#define ON 1;
#define OFF 0;

using namespace std;

typedef struct _ConvInstance
{
  char *infile;
  char *outfile;
  char *vfile;
  //  bool clock;

  bool ring;
  bool name;
  bool bracket;
  bool aroma4;
  bool b3D;
  bool index;
  bool cmol;
  bool aroma_color;
  bool skip;
  bool hachoalg;
  bool sssr;
  bool computer_A_ring;
  bool computer_C_ring;
  bool chiral;

  int limit;
  int metal;
  int ihy;
  int ih;
  int voption;
  int vout;
  int klist;
  int omol;

} ConvInstance;

int main(int argc , char *argv[])
{
  
  FILE *input_file;

  ConvInstance *param;
  param = (ConvInstance *)malloc( sizeof(ConvInstance)*1 );
  static molform mol;

  //  char temp[256];
  char opname[256] = {0};
  clock_t start_time,end_time;

  /*******************************
   *
   *  Get Option & In-Out Filename
   *
   ********************************/

  //  (*param).clock=false;

  (*param).ring=false;
  (*param).name=false;
  (*param).bracket=true;
  (*param).aroma4 = false;
  (*param).b3D = false;
  (*param).index = false;
  (*param).aroma_color = false;
  (*param).skip=false;
  (*param).hachoalg = false;
  (*param).sssr = false;
  (*param).computer_A_ring = true;
  (*param).computer_C_ring = true;

  (*param).limit = ON;
  (*param).metal = ON;
  (*param).ihy = ON;
  (*param).voption = OFF;
  (*param).vout = OFF;
  (*param).klist = OFF;

  (*param).cmol = false;
  (*param).omol = 0; // 0:output kcffile , 1:output molfile , 4: append aromaticity
  (*param).ih = false;
  (*param).chiral = false;

  
  bool vflg = false;

  //  cout << "argc = " << argc;
  if(argc==1){

    printf("\nCompound Kcf converter\n");
    printf("ex) >makekcf [options] [input file] [output file]\n\n");
    printf("Options\n");
    printf("\t\'-cmol\':\n");
    printf("\t\tCheck the number of the atoms and bonds without compute kcf\n");
    printf("\t\'-cmol0\':\n");
    printf("\t\tCheck the number of the atoms and bonds without compute kcf\n");
    printf("\t\treturn of the value '0' when the molfile have the atom \n\t\tand the bond.\n");
    printf("\t\'-omol\':\n");
    printf("\t\tOutput a molfile.\n");
    printf("\t\'-4\':\n");
    printf("\t\tOutput a kcffile/molfile which append the aromatic bond infomation '4'.\n"); 
    printf("\t\'-vr\':\n");
    printf("\t\tView ring status\n"); 
    printf("\t\t 0 non\n\t\t 1 member of the rings\n\t\t 2 member of the aromatic rings\n");  
    printf("\t\'-iH\':\n");
    printf("\t\tDisable of ignore explicit Hydrogen.\n");
    printf("\t\'-3D\':\n");
    printf("\t\tOutput kcf-file with the z-coordinate.\n");
    printf("\t\'-acolor\':\n");
    printf("\t\tAdd the color tag to the aromatic bonds.\n");
    printf("\t\'-sssr\':\n");
    printf("\t\tUse SSSR ring search algorithm.\n");
    //   printf("\t\'-hacho\':\n");
    //   printf("\t\tUse Dr Hattori's ring search algorithm.\n");
    printf("\t\'-limit\':\n");
    printf("\t\tEnable the calculating limit.\n");
    printf("\t\'-withoutmetal\':\n");
    printf("\t\tIgnore the metal atoms when calculate the ring atoms.\n"); 
    printf("\t\'-index\':\n");
    printf("\t\tDisplay the index table which is the difference between \n\t\tthe molfile and the kcffile.\n"); 
    printf("\t\'-vkcf\':\n");
    printf("\t\tDisplay the list of KEGG Atom\n");
    printf("\t\'-ihy\':\n");
    printf("\t\tnever consider the negative ion to the hydrogen.\n\t\t( ex. disable O(-)  => OH )\n");
    printf("\t\'-skip\':\n");
    printf("\t\tskip the calculating of the ring search and classify.\n");
    printf("\t\'-rskip\':\n");
    printf("\t\tskip the calculating of the ring search.\n");
    printf("\t\'-cskip\':\n");
    printf("\t\tskip the calculating of the complex ring search.\n");
    printf("\t\'-chiral\':\n");
    printf("\t\twrite #chiral when the chiral option of molfile equal 1.\n");
    printf("\t\'-voption\':\n");
    printf("\t\tmake a kcf vector representation.\n");
    printf("\t\'-vhoption\':\n");
    printf("\t\tmake a kcf vector representation which is expanded the Halogens\n"); 
    printf("\t\'-vout <filename>\':\n");
    printf("\t\tView a kcf vector representation.\n");
    printf("\t\'-name [name]\':\n");
    printf("\t\tset molecure name from [name]\n");
    return 0;
  }

  for(int i=1;i<argc;i++){

    switch(*argv[i]){
    case '-' :
      //      if( strcmp( argv[i] , "-time" ) == 0 )
      //	(*param).clock=true;
      if( strcmp( argv[i] , "-vr" ) == 0 )
	(*param).ring=true;
      if( strcmp( argv[i] , "-hacho" ) == 0 )
	(*param).hachoalg=true;
      if( strcmp( argv[i] , "-sssr" ) == 0 )
	(*param).sssr=true;
      if( strcmp( argv[i] , "-omol4" ) == 0 ) {
	(*param).omol=4;
      } else if( strcmp( argv[i] , "-omol" ) == 0 )
	(*param).omol=1;
      if( strcmp( argv[i] , "-limit" ) == 0 )
	(*param).limit=ON;
      if( strcmp( argv[i] , "-withoutmetal" ) == 0 )
	(*param).metal=OFF;
      if( strcmp( argv[i] , "-ihy" ) == 0 )
	(*param).ihy=OFF;
      if( strcmp( argv[i] , "-iH" ) == 0 )
	(*param).ih=true;
      if( strcmp( argv[i] , "-4" ) == 0 )
	(*param).aroma4=true;
       if( strcmp( argv[i] , "-acolor" ) == 0 )
	(*param).aroma_color=true;
      if( strcmp( argv[i] , "-3D" ) == 0 )
	(*param).b3D=true;
      if( strcmp( argv[i] , "-index" ) == 0 )
	(*param).index=true;
      if( strcmp( argv[i] , "-chiral" ) == 0 )
	(*param).chiral=true;
      if( strcmp( argv[i] , "-voption" ) == 0 )
	(*param).voption=1;
      if( strcmp( argv[i] , "-vhoption" ) == 0 )
	(*param).voption=2;
      if( strcmp( argv[i] , "-skip" ) == 0 )
	(*param).skip=true;
      if( strcmp( argv[i] , "-rskip" ) == 0 ) {
	(*param).computer_A_ring=false;
	(*param).computer_C_ring=false;
      }
      if( strcmp( argv[i] , "-cskip" ) == 0 ) {
	(*param).computer_C_ring=false;
      }
      if( strcmp( argv[i] , "-vout" ) == 0 ) {
	(*param).vout=1;
	if( argv[i + 1][0] != '-' ) {
	  i ++;
	  (*param).vfile = argv[i];
	}
      }

      if( strcmp( argv[i] , "-cmol0" ) == 0 ) {
	(*param).cmol=true;
	vflg = true;
      } else if( strcmp( argv[i] , "-cmol" ) == 0 ) {
	(*param).cmol=true;
	vflg = false;
      }
      
      if( strcmp( argv[i] , "-name" ) == 0 ) {
	(*param).name = true;
	if( argv[i + 1][0] != '-' ) {
	  i ++;
          sprintf(opname,"%-26s",argv[i]);
	}
      }

      if( strcmp( argv[i] , "-vkcf" ) == 0 )
	(*param).klist=1;
      break;

    default: //FileName 1stColum:Input 2ndColum:Output

      (*param).infile = argv[i];
      i++;
      (*param).outfile = argv[i];
    }
  }
 
  if ( (*param).omol != 0 ) {
    (*param).ih = true;
  }

  if(  (*param).cmol ) {

    if((input_file = fopen((*param).infile,"r")) == NULL) {
      if( vflg ) {
	cout << 1;
      } else {
	printf("file:%s not exist.\n",(*param).infile);
      }
      exit(9);
    } else {
      mol = read_Molfile(input_file , false );      
      fclose(input_file);
    }
    if( mol.nodenum != 0 ) {
      if( vflg ) {
	cout << 0;
      } else {
	cout <<  mol.nodenum << ":" << mol.bondnum << endl;
      }
    } else {
      if( vflg ) {
	cout << 2;
      } else {
	cout <<  "error node = 0 " << mol.bondnum << endl;
      }
    }
    exit(9);
  }

  kcflist klist;
  klist = SetKcfList(0);
 
  if ( (*param).klist > 0 ) {
    print_Klist(klist);
  }

  if((*param).infile==NULL){
    printf("Not found:Input file name.\n");
    exit(9);
  }else if((*param).outfile==NULL){ 
    printf("Not found:Output file name.\n");
    exit(9);
  }
  //  cout << "(?_?)" << endl;
  if((*param).vout != 0 && (*param).voption == 0 )
    (*param).voption = 1;

  /********************************/

  start_time = clock();
  brackets brinfo;
  brinfo.member = 0;

  int file_type = 0; // file_type == 0 : mol , file_type == 1 , kcf
 
  if( strstr( getstr_line(  (*param).infile ,  strlen( (*param).infile) - 4 ,  strlen( (*param).infile)  ) , ".kcf" ) ) {
    file_type = 1;
  }

  if((input_file = fopen((*param).infile,"r")) == NULL) {
    printf("file:%s not exist.\n",(*param).infile);
    exit(9);
  } else {

    if( file_type == 0 ) {
      mol = read_Molfile(input_file , (*param).ih );      
      if( (*param).bracket ) {
	fseek(input_file,1,SEEK_SET);
	brinfo = read_Bracket(input_file);  
      }
    } else {
      mol = read_Kcffile( input_file , (*param).ih );      
    }
    fclose(input_file);
  }

  /* add 090924 */
/*--
  mol.oname =  getstr_line((*param).infile,strlen((*param).infile) - (5+26) ,strlen((*param).infile) - 5 );
--*/

  int ppos = 0;
  int epos = 0;
  int snum = strlen((*param).infile);

  for ( int i= snum -1 ; i >= 0 ; i-- ){
#ifdef DEBUG
printf("oname = %c\n",(*param).infile[i]);
#endif
    if ( (*param).infile[i] == '.' && epos == 0){
      epos = i-1;
    }
    if ( (*param).infile[i] == '/' ){
      ppos = i+1;
      break;
    }
  }

  if ( epos == 0 ) epos = snum;

#ifdef DEBUG
printf("ppos = %d : epos = %d\n",ppos,epos);
#endif
  if ( epos - ppos > 26 ) epos = ppos +26;

  mol.oname =  getstr_line((*param).infile,ppos ,epos );

#ifdef DEBUG
printf("oname = %s\n",mol.oname);
#endif
/* add 090924 ・ム・ケ、鮑 */

  if( (*param).chiral && mol.chiral == 1 ) {
    mol.chiral = 1;
  } else {
    mol.chiral = 0;
  }

  /* kcf ENTRY */
  if( (*param).name ) {
    if( strlen( opname ) > 26 ) {
      mol.name = getstr_line( opname , 0 , 26 );
    } else {
      mol.name = opname;
    }
  } else {
    mol.name = mol.oname;
  } 
  mol.ihy = (*param).ihy;

  if(  mol.nodenum == 0 ) {
    printf("illegal molfile:%s \n",(*param).infile);
    exit(9);
  }

  //  int ringSize = 4;
  int ringSize = int( mol.nodenum / 2) + 1;

  int iSSSR = mol.bondnum - mol.nodenum + 1;
  int iISO = 0;
  int iCOM = 0;

  int **cmat;
  cmat = matrix_new(mol.nodenum, mol.nodenum); // make a new connection matrix 
  matrix_zero(cmat,mol.nodenum); // set 0 to the matrix

  // set the values to the matrix from the information.
  if( (*param).metal == 0 ) {
    //    cout << "make a connection matrix without metal." << endl;
    set_cmatrix(cmat,mol,1,1); 
  } else {
    //    cout << "make a connection matrix." << endl;
    set_cmatrix(cmat,mol,1,0); 
  }

  matrix_delete_terminalnode(cmat, mol.nodenum);  
  // make the connection paires from the connection matrix.

  int **cpair;

  int edges = count_edges(cmat, mol.nodenum );
  cpair = set_cpair(cmat, mol.nodenum);
  
  //  for( int i=0; i< edges; i ++ )
  //    cout << i << ':' << cpair[i][0] << ' ' << cpair[i][1] << endl;
  // cout << endl;
  //  cout << "-- end make cmat --" << endl;

  int **cmat_b;
  // make a connection matrix with bonds infomation.
  cmat_b = matrix_new(mol.nodenum, mol.nodenum);
  matrix_zero(cmat_b,mol.nodenum);
  set_cmatrix(cmat_b,mol,0,0);

  int *cindex = new int [mol.nodenum];
  for(int i=0 ; i < mol.nodenum; i++ ) {
    cindex[i] = i;
  }

  //  cout << "cindex_edge " << edges << " " << mol.bondnum << endl;

  int *cindex_edge = new int [mol.bondnum]; // index_edge 配列の生成
  cindex_edge = set_edge_index( mol , cpair , edges );

  //  for(int i=0 ; i< mol.bondnum  ; i ++ )
  //    cout << i << ':' << cindex_edge[i] << endl;


  int *ring_flg = new int [mol.nodenum]; // ring_flg配列の生成
  for(int i=0 ; i < mol.nodenum; i++ ) {
    ring_flg[i] = 0;
  }

  //  cout << "a" << endl;

  int **computed_edges; // 計算済のエッジ [0][0]はエッジ数
  computed_edges = matrix_new(mol.bondnum + 1 , 2 );
  computed_edges[0][0] = 0;
  computed_edges[0][1] = 0;

  //  cout << "b" << endl;

  if( (*param).skip ) {
    if( file_type == 0 ) {
      for(int i=0 ; i < mol.nodenum; i++ ) 
	mol.kcfsymbol[i] = mol.atomsymbol[i];     
    }

  } else {

    //    cout << "(?0?)" << endl;
    // ringを探索する。

    if( ! (*param).computer_A_ring ) 
      cout << "the Ring search proccess is skiped\n";

    if( mol.bondnum > 1 && (*param).computer_A_ring ) {

      pathlist isorings;
      isorings.count = 0;
    
      //      int iso_roop = 0;
      //      int isorings_num = 0;
  
      int edge_flg = 1;
	
      if( (*param).sssr ) {

	// SSSR algo
	isorings = search_SSSR_rings( cmat , cindex , mol.nodenum , isorings , ringSize );
	edge_flg = 0;

      } else if( (*param).hachoalg ) {

	// hacho algo

	isorings = search_isorings(cmat , cindex , mol.nodenum ,  0 , computed_edges, isorings );	
	edge_flg = 0;

      } else {

	isorings = search_isorings_depth(cpair , mol.nodenum , edges , isorings , edge_flg);

      }      

      /// cout << "isoend" << endl;

      // mol ファイルが元から aromaticity をもっているかチェック

      bool aroma_bond = false;
      aroma_bond = check_aromabond(mol);
      
      // isoring間の接続状況をカウントする

      if( isorings.count > 0 ) {
	int **r_status;
	r_status = matrix_new( isorings.count  , isorings.count );

	int *iso_aroma_status;
	iso_aroma_status = index_new( isorings.count );

	for(int i=0 ; i < isorings.count ; i ++) {
	  for(int j=i ; j < isorings.count ; j ++) {  
	    r_status[i][j] = diff_path( isorings.list[i] , isorings.list[j] );
	    r_status[j][i] = r_status[i][j];
	    //	    cout << r_status[i][j] << " "; 
	  }
	  //	  cout << endl;
	}
	
	// isoring情報を元に、ringか否かの情報を設定してしまう。
	
	for(int i=0 ; i < isorings.count ; i ++) {
	  for(int j=1 ; j <= isorings.list[i][0] ; j ++) { 
	    
	    if( edge_flg == 0 ) {	    
	      // リング情報をノードで扱いたい場合はこっち
	      ring_flg[isorings.list[i][j]] = 1;
	      
	    } else {
	      
	      // リング情報をエッジで扱いたい場合はこっち 
	      ring_flg[ cpair[ isorings.list[i][j] ][0] ] = 1;
	      ring_flg[ cpair[ isorings.list[i][j] ][1] ] = 1;  
	      
	    }
	  }
	}

	// isoringに対してhukkel則の計算をする。

	if( ! aroma_bond ) {
	  // aroma_bond が存在していた場合は hukel および complex_ring の探索をしない

	  for(int i=0 ; i < isorings.count ; i ++) {
	    //	  cout << "<huckel : " << i << ">" << endl; 
	    int huckel;
	    
	    if( edge_flg == 0 ) {
	      //	    huckel = huckel_check( mol , cmat_b ,isorings.list[i] );
	      huckel = huckel_check( mol , cmat_b ,isorings.list[i] , cpair , 0 );
	    } else {
	      //	    huckel = huckel_check_edge( mol , cmat_b ,isorings.list[i] , cpair );
	      huckel = huckel_check( mol , cmat_b ,isorings.list[i] , cpair , 1);
	    }
	    
	    if( huckel != 0 ) {
	      //	    cout << "huckel = " << huckel << endl;
	      
	      iso_aroma_status[i] = 1;
	      
	      if( edge_flg == 0 ) {
		// リング情報をノードで扱いたい場合はこっち
		for(int j=1 ; j <= isorings.list[i][0] ; j ++) {
		  ring_flg[isorings.list[i][j]] = 2;
		  
		}
		for(int k=0 ; k < mol.bondnum ; k++ ) {
		  if( grep_node(mol.bond.s[k] - 1, isorings.list[i] ) == 1 && grep_node(mol.bond.e[k]  - 1 , isorings.list[i] ) == 1 ) 
		    mol.bond.ring[k] = 2;
		}
		
	      } else {
		// リング情報をエッジで扱いたい場合はこっち 
		for(int j=1 ; j <= isorings.list[i][0] ; j ++) {
		  
		  ring_flg[ cpair[ isorings.list[i][j] ][0] ] = 2;
		  ring_flg[ cpair[ isorings.list[i][j] ][1] ] = 2;  
		  
		  int m_index = move_index(  isorings.list[i][j] ,cindex_edge ,  mol.bondnum , -1) ;
		  if( m_index > -1 ) 
		    mol.bond.ring[m_index] = 2;
		  
		}
	      } // edge_flg == 0 
	    } else {
	      iso_aroma_status[i] = 0;
	    } // huckel == 0 
	  }
	}
	
	iISO = isorings.count;
	
	// isoringの一覧を見るっす。
	if( (*param).ring ) {
	  cout << mol.name << " isolated rings = " << isorings.count << endl;
	  for(int i=0 ; i < isorings.count ; i ++) {
	    cout << "path=" << i << "/" <<  isorings.list[i][0] ;

	    if( iso_aroma_status[i] == 0 ) {
	      cout << " (N): ";
	    } else {
	      cout << " (A): ";
	    }
	    for(int j=1 ; j <= isorings.list[i][0] ; j ++) {
	      cout << isorings.list[i][j] << " ";
	    }
	    cout << ":" << isorings.list[i][isorings.list[i][0] + 1   ] ;
	    cout << endl;
	  }
	}
	
	int hflg = 0;
	for(int i=0 ; i < mol.nodenum ; i ++) {
	  if( ring_flg[i] == 1 )
	    hflg = 1;
	}
	
	//      hflg = 0;
	
	if( ! aroma_bond ) {
	  if( ( (*param).limit == 1 && isorings.count > 200) && hflg == 1) {
	    
	    // hflg == 1 のとき、complex_ringを求める。
	    cout << mol.name << ": abandon computer : isoring = " << isorings.count << endl;
	    
	  } else if( hflg == 1  ) {
	    
	    if( ! (*param).computer_C_ring ) {
	      cout << "the complex Ring search proccess is skiped\n";
	    } else {

	      if( (*param).ring ) {
		cout << "compute complex_ring" << endl;
	      }
	      // numは総最大リング数
	      pathlist complex_rings;
	      
	      complex_rings.count = 0;
	      
	      if( edge_flg == 0 ) {
		
		//		cout << "L" << endl;
		complex_rings = search_complexrings( isorings , complex_rings, mol.nodenum + 1 ,0  );
		//		cout << "L2" << endl;
		set_outline( complex_rings , cpair , mol.nodenum  , edges );
		
	      } else {
		
		//		cout << "R" << endl;
		complex_rings = search_complexrings_edge( isorings , complex_rings, mol.bondnum  , 0  );
		
	      }
	      	      
	      int *cmp_aroma_status;
	      cmp_aroma_status = index_new( complex_rings.count );
	      
	      // complex_ringに対してhukkel則の計算をする。
	      //      cout << "complex_rings.count = " << complex_rings.count << endl;
	      
	      for(int i=0 ; i < complex_rings.count ; i ++) {
		//	    cout << "<huckel : " << i << ">" << endl; 
		int huckel;
		
		if( edge_flg == 0 ) {
		  //	      huckel = huckel_check( mol , cmat_b , complex_rings.list[i] );
		  huckel = huckel_check( mol , cmat_b , complex_rings.list[i] , cpair , 0);
		} else {
		  //	      huckel = huckel_check_edge( mol , cmat_b , complex_rings.list[i] , cpair );
		  huckel = huckel_check( mol , cmat_b , complex_rings.list[i] , cpair , 1);
		}
		
		if( huckel != 0 ) {	 
		  
		  cmp_aroma_status[i] = 1;
		  //	      cout << "huckel = " << huckel << endl;
		  
		  if( edge_flg == 0 ) {
		    // リング情報をノードで扱いたい場合はこっち
		    for(int j=1 ; j <= complex_rings.list[i][0] ; j ++) {
		      ring_flg[ complex_rings.list[i][j]  ] = 2;
		      
		    }
		    for(int k=0 ; k < mol.bondnum ; k++ ) {
		      if( grep_node(mol.bond.s[k] - 1, complex_rings.list[i] ) == 1 && grep_node(mol.bond.e[k]  - 1 , complex_rings.list[i] ) == 1 ) 
			mol.bond.ring[k] = 2;
		    }
		    
		  } else {
		    // リング情報をエッジで扱いたい場合はこっち 
		    for(int j=1 ; j <= complex_rings.list[i][0] ; j ++) {
		      
		      ring_flg[ cpair[ complex_rings.list[i][j] ][0] ] = 2;
		      ring_flg[ cpair[ complex_rings.list[i][j] ][1] ] = 2;  
		      
		      int m_index = move_index( complex_rings.list[i][j] ,cindex_edge ,  mol.bondnum , -1) ;
		      if( m_index > -1 ) 
			mol.bond.ring[m_index] = 2;
		      
		    }
		  } // edge_flg == 0 
		} else {
		  cmp_aroma_status[i] = 0;
		}
	      }
	  
	      iCOM = complex_rings.count;	  
	      
	      if ((*param).ring) {
		cout << mol.name <<  " complexed rings = " << complex_rings.count << endl;
		for(int i=0 ; i < complex_rings.count ; i ++) {
		  cout << "path=" << i << "/" << complex_rings.list[i][0];
		  
		  if( cmp_aroma_status[i] == 0 ) {
		    cout << " (N): ";
		  } else {
		    cout << " (A): ";
		  }
		  
		  for(int j=1 ; j <= complex_rings.list[i][0] ; j ++) {
		    cout << complex_rings.list[i][j] << " ";
		  }
		  cout << ":" << complex_rings.list[i][complex_rings.list[i][0] + 1   ] ;
		  cout << endl;
		}
	      }
	    }
	  }
	} else {
	 
	  cout << mol.name << " : the aromaticity was already calculated." << endl;
	  for(int i=0 ; i < mol.bondnum ; i ++) {
	    if( mol.bond.bondtype[ i ] == 4 ) {
	      mol.bond.ring[i] = 2;
	  
	      ring_flg[ mol.bond.s[i] - 1] = 2;
	      ring_flg[ mol.bond.e[i] - 1] = 2;
	    }
	    
	  }
	  
	}

	if ((*param).ring) {
	  cout << "rings status= ";
	  for(int i=0 ; i < mol.nodenum ; i ++) {
	    if( ring_flg[i] >= 1 )
	      cout << i << "(" << ring_flg[i] << ") ";
	  }
	  cout << endl;
	}
      }
    }

    // vector 表示 オプションを追加した。

    int *kcfvector;
    int kcfdim;
    
    if( (*param).voption == 1 ) {
      kcfdim = klist.num_c +  klist.num_n + klist.num_o + klist.num_s + klist.num_p + 2;
      //    cout << "dim = " << kcfdim << endl;
      kcfvector = index_new(kcfdim);
      
    } else if( (*param).voption == 2 ) {
      kcfdim = klist.num_c +  klist.num_n + klist.num_o + klist.num_s + klist.num_p + klist.num_x + 1;
      //     cout << "dim = " << kcfdim << endl;
      kcfvector = index_new(kcfdim);
    }
    
    for(int i=0 ; i < mol.nodenum; i++ ) {
      
      if( mol.atomsymbol[i] == "R" ) {
	mol.kcfsymbol[i] = "R";
	
      } else if( mol.atomsymbol[i] == "F" || mol.atomsymbol[i] == "Cl" ||  mol.atomsymbol[i] == "Br" || mol.atomsymbol[i] == "I" ) {
	mol.kcfsymbol[i] = "X";
	
      } else if (mol.atomsymbol[i] == "C") {	
	
	//      cout << "---C---" << endl;
	if( ring_flg[i] > 0 ) 
	  mol.kcfsymbol[i] = classify_C_ring(mol,cmat_b,ring_flg,i);
	else
	  mol.kcfsymbol[i] = classify_C(mol,cmat_b,i);
	
      } else if (mol.atomsymbol[i] == "N") {	
	
	//      cout << "---N---" << endl;
	if( ring_flg[i] > 0 ) 
	  mol.kcfsymbol[i] = classify_N_ring(mol,cmat_b,ring_flg,i);
	else
	  mol.kcfsymbol[i] = classify_N(mol,cmat_b,i);
	
      } else if (mol.atomsymbol[i] == "O") {	
	
	//      cout << "---O---" << endl;
	if( ring_flg[i] > 0 ) 
	  mol.kcfsymbol[i] = classify_O_ring(mol,cmat_b,ring_flg,i);
	else
	  mol.kcfsymbol[i] = classify_O(mol,cmat_b,ring_flg,i);
	
      } else if (mol.atomsymbol[i] == "S") {	
	
	//      cout << "---S---" << endl;
	mol.kcfsymbol[i] = classify_S(mol,cmat_b,ring_flg,i);
	
      } else if (mol.atomsymbol[i] == "P") {	
	
	//      cout << "---P---" << endl;
	mol.kcfsymbol[i] = classify_P(mol,cmat_b,i);

      } else if (mol.atomsymbol[i] == "H") {	
	
	//      cout << "---P---" << endl;
	mol.kcfsymbol[i] = "H";

/* add 20090916 start */
      } else if (mol.atomsymbol[i] == "X") {
	//      cout << "---X---" << endl;
  mol.kcfsymbol[i] = "X";
/* add end */
      } else {
	mol.kcfsymbol[i] = "Z";
      }
      
      if( (*param).voption > 0 ) {	
	    kcfvector = SetKcfVector( kcfvector , mol.kcfsymbol[i] , 
          mol.atomsymbol[i] , klist , (*param).voption );
      }
    }

    // add 20091027 start
    ModifyKcfSymbol(&mol, cmat_b);
    // add end
    
    if ((*param).aroma4) {
      cout << mol.name << ": set aromatic \"4\" to bond infomation." << endl;  

      for(int i=0 ; i < mol.nodenum ; i ++) {
	if( ring_flg[ mol.bond.s[i] - 1 ] == 2 && ring_flg[ mol.bond.e[i] - 1 ] == 2 ) {   
	  cout << i << " (" << mol.bond.s[i] << "-" << mol.bond.e[i] << ") ";
	  mol.bond.bondtype[i] = 4;  
	}
      }
      cout << endl;
    }

    if( (*param).voption > 0 ) {
      
      if( (*param).vout == 1 ) 
	output_KcfVector_line( kcfvector , kcfdim , mol.name, (*param).vfile);
      
      index_del(kcfvector);
    }
    if ( (*param).index ) {
      cout << "INDEX(" << mol.name <<  ")=";
      for( int i = 1 ; i <= mol.index[0] ; i ++ ) {
	cout << mol.index[i] << " ";
      }
      cout << endl;
    }
  }

  if ( (*param).outfile != NULL ) {
    if( (*param).omol == 4 ) {
      output_mol_aromaticity((*param).infile, (*param).outfile ,ring_flg);
    } else if( (*param).omol == 1 ) {
      output_Molfile(mol , (*param).outfile );
    } else {
      output_kcffile(mol, brinfo, (*param).outfile , (*param).b3D , (*param).aroma_color );
    }
  }

  end_time = clock();

  double d_time;
  //  d_time = (end_time - start_time) / CLOCKS_PER_SEC;
  d_time = (double)(end_time - start_time)/(double)CLOCKS_PER_SEC;

  cout << mol.name << " SSSR=" << iSSSR << " isolated-rings=" << iISO << " comprexed-rings=" << iCOM << " computed_time=" << d_time << "sec." << endl;

  //  if( (end_time - start_time)/CLOCKS_PER_SEC > 10 || (*param).clock ) 
  //    cout << mol.name << " " << (end_time - start_time)/CLOCKS_PER_SEC << "sec."<< endl;

  free(param);
}

