// search_rings.c
//
// 16 January 2009 T.Ohya
// 24 August 2004 K.Tonomura
//

#include "search_rings.h"

int gloval_roop = 0;

// ring search subroutine

// haccyo アルゴリズムでサーチ
pathlist search_isorings(int **connection_matrix, int *connection_index, int nodenum, int loop, int **computed_edges , pathlist isorings ) {

  pathlist isorings_wk;
  int i,j,dim,count;
  int local_loop = loop;
  isorings_wk = isorings;
  
  gloval_roop ++;
  local_loop ++;

  int **clique;
  clique = matrix_copy(connection_matrix,nodenum ,nodenum);

  int *cliqueindex;
  cliqueindex = index_new(nodenum);

  for( i=0; i<nodenum; i++)
    cliqueindex[i] = connection_index[i];

  //  matrix_delete_isolatednode(clique, nodenum);
  matrix_delete_terminalnode(clique, nodenum);
  dim =  matrix_degeneracy(clique, cliqueindex , nodenum , 0) ;   

  // isolated ring を取りだす。
  //// エッジ数3以上のノードのリストを抽出
  //// hubs[0]はHub数 hubs[hubs[0]+1]は総エッジ数

  int **hubs;
  hubs = search_hubnode(clique,dim,1); 

  int *hubs_path;
  hubs_path = index_new(dim+1);
  search_pathway(hubs_path,hubs[0],clique , dim);

  int *c_hubs;  
  c_hubs = complymentary_set(hubs_path,dim); // hubsの補集合を作成 この集合で作られるパス群はisolated ringを形成する。

  if( c_hubs[0] != 0 ) {

    int *tmp_path;
    tmp_path = index_new(dim+1);
    tmp_path[0] = 0;

    for(i = 1; i<=c_hubs[0] ; i ++ ) {

      if(grep_node(c_hubs[i],tmp_path) == 0 ) {

	int *isoring_point_path;
	isoring_point_path = index_new(nodenum + 2);

	isoring_point_path[0] = 1;
	isoring_point_path[1] = c_hubs[i];
	count = 100;
	while( count != 0){
	  count = add_neighbor_node(isoring_point_path,clique,dim);
	}
	add_node(isoring_point_path,tmp_path);

	int *isoring_point_path_i;
	
	isoring_point_path_i = index_new(nodenum + 2);
	correspond_index(isoring_point_path_i,isoring_point_path,cliqueindex);

	isoring_point_path_i[ isoring_point_path_i[0] + 1 ] = isoring_point_path_i[0]; // set outline infomation

	//	sort_node(isoring_point_path_i);
	normalyse_node(isoring_point_path_i);


	if( path_grep(isoring_point_path_i,isorings_wk) == -1 ) {
	  if(isorings_wk.count == 0 ) {
	    isorings_wk.count = 1;
	    isorings_wk.list = (int**)malloc(isorings_wk.count*sizeof(int*));
	  }  else {
	    isorings_wk.count ++;
	    isorings_wk.list = (int**)realloc(isorings_wk.list,isorings_wk.count*sizeof(int*));
	  }
	  if( isorings_wk.list == 0 ) {
	    cout << "メモリ領域の確保に失敗しました : " << isorings_wk.count << endl;
	    exit(1);
	  } else {
	    isorings_wk.list[isorings_wk.count - 1] = isoring_point_path_i;
	  }
	}
	//	index_del(isoring_point_path_i);
	index_del(isoring_point_path);
      }
    } 
    index_del(tmp_path);
  }

  index_del(c_hubs);
  index_del(hubs_path);

  // Hubポイントの枝を落して、iso_pathを計算します。
  if(  hubs[1][0] > 0) {
    
    int **hubs_edges;
    hubs_edges = matrix_new( hubs[1][0] + 1,2);
  
    hubs_edges[0][0] = 0;
    hubs_edges[0][1] = 0; 
    // 配列[0][1]は使っていないけど、nullが入ってると不都合が出るかも知れないので保険
    
    for(i = 1; i<=hubs[0][0]; i++) {
      for(j = 0; j < dim; j++ ) {
	if(j != hubs[0][i] && clique[j][hubs[0][i]] != 0) {

	  // 計算済エッジかどうか判定し、計算済ならadd_edgesしない
	  if( check_finished_edge(computed_edges,cliqueindex,hubs[0][i],j) == 0 ) {
	    add_edge(hubs_edges,hubs[0][i],j);
	  }
	}
      }
    }
    int *sub_cliqueindex;
    sub_cliqueindex = index_new(dim);

    for(i = 0; i< dim; i++ )
      sub_cliqueindex[i]  = cliqueindex[i];

    int **sub_clique;
    for(i = 1; i<=hubs_edges[0][0]; i++ ) {

      // loop深度が1以下の場合に計算済エッジに加える
      if(local_loop <= 1 ) {
	add_edge(computed_edges,cliqueindex[hubs_edges[i][0]],cliqueindex[hubs_edges[i][1]]);
      }

      sub_clique = matrix_copy(clique,dim,dim);
      //      int dis = delete_edge(hubs_edges[i],sub_clique, dim);      
      //      printf("<loop=%d:edges=%d/%d /// delete edges (%d %d) /// dim = %d>\n",local_loop,i,hubs_edges[0][0],hubs_edges[i][0],hubs_edges[i][1],dim);

      if( gloval_roop > maximum_roop )
	cout << "roop over flow " << gloval_roop << endl;
      else
	isorings_wk = search_isorings(sub_clique, sub_cliqueindex, dim , local_loop , computed_edges, isorings_wk );

      //      printf("<loopend=%d:edges=%d/%d /// delete edges (%d %d) /// dim = %d>\n",local_loop,i,hubs_edges[0][0],hubs_edges[i][0],hubs_edges[i][1],dim);
      matrix_del(sub_clique,dim);

    }
    index_del(sub_cliqueindex);

    matrix_del(hubs_edges,hubs[1][0]+1);
    //    for(i=0; i <= hubs[1][0] ; i++ ) 
    //      delete [] hubs_edges[i];
    //   delete [] hubs_edges;
  }
  matrix_del(hubs,2);
  matrix_del(clique,nodenum);
  index_del(cliqueindex);
  //  printf("<return %d:%d>\n",loop,dim);
  return isorings_wk;

}

////////////////////////////////////////////////////////////////

int *getRing_SSSR(int init_node , int **connection_matrix , int dim , int ringSize ) {

  int *rRing;
  rRing = index_new(dim + 2);
  rRing[0] = 0;

  //  return rRing;
  //  cout << "getRing_SSSR" << endl;

  pathlist r_list;

  int *seed_path;
  seed_path = index_new(dim + 1);
  seed_path[0] = 1;
  seed_path[1] = init_node;
  
  r_list.count = 1;
  r_list.list = (int**)malloc(r_list.count*sizeof(int*));  
  r_list.list[0] = seed_path;

  int r_new = 0;
  int depth = 1;

  int start , end;
  start = 0; end = 0; 
  int *neighbor;

  while( r_new == 0 && depth < ringSize ) {
 
    for( int i = start ; i <= end ; i++ ) {
      neighbor = index_new( dim + 1 );
      neighbor[0] = 0;

      for( int j = 0 ; j < dim ; j ++ ) {
	if( depth == 1 ) {
	  if( connection_matrix[ init_node ][j] != 0 &&  init_node != j ) {
	    neighbor[0] ++;
	    neighbor[ neighbor[0] ] = j;
	  }
	} else {
	  if( connection_matrix[ r_list.list[ i ][depth] ][j] != 0 &&  r_list.list[ i ][depth] != j && r_list.list[ i ][depth - 1] != j  ) {
	    neighbor[0] ++;
	    neighbor[ neighbor[0] ] = j;
	  }
	}
      }
    
      if( neighbor[0] > 0 ) {
	for( int j = 1 ; j <= neighbor[0] ; j ++ ) {

	  if( grep_node( neighbor[j] , r_list.list[i] ) == 0 ) {

	    r_list.count ++;
	    r_list.list = (int**)realloc(r_list.list,r_list.count*sizeof(int*));	  

	    int *wk_path;
	    wk_path = index_copy( r_list.list[i] , dim );
	    wk_path[0]  =   wk_path[0] + 1;
	    wk_path[  wk_path[0] ] = neighbor[j];

	    r_list.list[ r_list.count - 1 ] = wk_path; 
	
	    //	    cout << "add_path[" << r_list.count - 1 << "] " << neighbor[j] << "=" ;
	    for( int k = 1 ; k <= wk_path[0] ; k ++ ) {
	      //	      cout << wk_path[k] << " ";
	    }
	    //	    cout << endl;
	  }
	}

      } else {
	r_new = 2;
      }
    }

    if( end < r_list.count ) {
      //      cout << "-------- end --------" << endl;
      start = end + 1;
      end =  r_list.count - 1 ;

    } else {
      r_new = 2;
    }

    //    cout << "start=" << start << " end=" << end << " neighbor=" << neighbor[0] << " depth=" << depth << " r_list.count=" << r_list.count << endl; 

    if( depth > 0 &&  r_new == 0 ) {
      for( int i = start ; i <= end && r_new == 0 ; i++ ) {
	for( int j = i + 1 ; j <= end && r_new == 0 ; j++ ) {
	  //	  cout << "   i:j=" << i << ":" << j << endl;
	  if( r_list.list[i][2] !=  r_list.list[j][2] ) {
	    
	    int match = 0;
	    
	    if( r_list.list[i][r_list.list[i][0] ] == r_list.list[j][r_list.list[j][0] ]  ) {
	      match = 2;
	      //      cout << "even" << endl;
	    } else if ( connection_matrix[r_list.list[ i][r_list.list[i][0] ] ][r_list.list[j][r_list.list[j][0] ] ] != 0 ) { 
	      match = 1;
	      //	      cout << "odd" << endl;
	    } 

	    if( match != 0 ) {
	      // even members ring || odd members ring
	      int m;
	      for( m = 1 ; m <= r_list.list[i][0] ; m++ ) {
		rRing[0] ++;
		rRing[m] =  r_list.list[i][m];
		//		cout <<  r_list.list[i][m] << " ";
	      }
	      //	      cout << "(" << rRing[0]  << ")";
	      //	      cout <<  "| "; 
	      for( m = r_list.list[j][0] - match + 1 ; m > 1 ; m-- ) {
		rRing[0] ++;
		rRing[ rRing[0] ] =  r_list.list[j][m];
		//		cout <<  r_list.list[j][m] << " ";
	      }
	      //	      cout << "(" << rRing[0]  << ")";
	      //	      cout << endl;
	      rRing[ rRing[0] + 1 ] = rRing[0]; // set outline infomation 
	      r_new = 1;
	      i = end;
	      j = end;
	    }
	  }
	}
      }
    }
    
    //    cout << "depth = " << depth << " (" << ringSize << ")" << endl; 
    depth ++;
  }

  /*
  cout << "rRing(" << rRing[0] << ")= ";
  for( int i=1 ; i<= rRing[0] ; i++ ) 
    cout << rRing[i] << " " ;
  cout << endl;
  */  

  free(r_list.list);
  index_del(seed_path);

  return rRing;

}

pathlist search_SSSR_rings(int **connection_matrix, int *connection_index, int nodenum, pathlist isorings , int ringSize ) {

  pathlist isorings_wk;
  isorings_wk = isorings;
  int i , j;

  //  cout << "SSSR" << endl;

  int **clique;
  clique = matrix_copy(connection_matrix,nodenum ,nodenum);

  int *cliqueindex;
  cliqueindex = index_new(nodenum);
  for(i=0; i<nodenum; i++)
    cliqueindex[i] = connection_index[i];

  int *fullSet;
  fullSet = index_new(nodenum + 1);
  fullSet[0] = nodenum;

  for(i=1 ; i <= nodenum; i++)
    fullSet[i] = 1;

  int *trimSet;
  trimSet = index_new(nodenum + 1);

  trimSet[0] = nodenum;
  for(i=1 ; i <= nodenum; i++)
    trimSet[i] = -1;

  int min_digree;
  int init_state;

  int *nodesN2;

  do {

    min_digree = 100;
    init_state = -1;
    int iline;

    nodesN2 = index_new( nodenum + 1 );
    nodesN2[0] = 0;

    for(i=0; i < nodenum ; i ++ ) {
      iline = 0;
      for(j=0; j< nodenum ; j ++ ) {
	if( clique[i][j] != 0 ) 
	  iline ++;
      }
      if( iline < min_digree && iline > 0) {
	min_digree = iline;
	init_state = i;
      }
    }
    if( min_digree == 100 ) 
      min_digree = 0;
    
    matrix_delete_terminalnode(clique , nodenum );

    for( i = 0 ;  i < nodenum; i++) {
      if( count_edge( i , clique , nodenum ) <= 1 ) {
	trimSet[ i + 1 ] = 1; 
      }
    }

    //    cout << "douda!" << endl;

    for(i=0; i< nodenum ; i ++ ) {

      iline = 0;
      for(j=0; j< nodenum ; j ++ ) {
	if( clique[i][j] != 0 ) 
	  iline ++;
      }
      if( iline == 2 && grep_node(i,trimSet) == 0 ) {
	nodesN2[0] ++;
	nodesN2[ nodesN2[0] ] = i;
      }
    }
    
    if( min_digree == 2 ) {

      for( i=1 ; i <= nodesN2[0] ; i ++ ) {

	int *isoring_point_path_i;	
	
	isoring_point_path_i = getRing_SSSR( nodesN2[i] , clique , nodenum , ringSize ); 

	//	cout << "-Ring(" << isoring_point_path_i[0] << ")= ";
	//	for( int j=1 ; j <= isoring_point_path_i[0] ; j++ )
	//	  cout << isoring_point_path_i[j] << " ";
	//	cout << endl;
	   
	//	sort_node(isoring_point_path_i);

	normalyse_node(isoring_point_path_i);

	//	cout << "rRing(" << isoring_point_path_i[0] << ")= ";
	//	for( int j=1 ; j<= isoring_point_path_i[0] ; j++ ) 
	//	  cout << isoring_point_path_i[j] << " " ;
	//	cout << endl;

	if( isoring_point_path_i[0] != 0  && path_grep(isoring_point_path_i,isorings_wk) == -1 ) {
	  if(isorings_wk.count == 0 ) {
	    isorings_wk.count = 1;
	    isorings_wk.list = (int**)malloc(isorings_wk.count*sizeof(int*));
	  }  else {
	    isorings_wk.count ++;
	    isorings_wk.list = (int**)realloc(isorings_wk.list,isorings_wk.count*sizeof(int*));
	  }
	  if( isorings_wk.list == 0 ) {
	    cout << "メモリ領域の確保に失敗しました : " << isorings_wk.count << endl;
	    exit(1);
	  } else {
	    isorings_wk.list[isorings_wk.count - 1] = isoring_point_path_i;
	    //	    cout << "isorings_wk.count == " <<  isorings_wk.count << endl;
	  }
	} else {
	  index_del(isoring_point_path_i);
	}
	delete_node( nodesN2[i] , clique , nodenum );    
	matrix_delete_terminalnode(clique , nodenum );
      }

      //      for( i=1 ; i <= nodesN2[0] ; i ++ ) { 
      //	delete_node( nodesN2[i] , clique , nodenum );      
      //      }

    } else if ( min_digree == 3 ) {

      ///// SSSRの欠点はこういうところらしい。

      int *isoring_point_path_i;	
      //      isoring_point_path_i = index_new(index_node + 1);

      isoring_point_path_i = getRing_SSSR( init_state , clique , nodenum , ringSize ); 
      //      sort_node(isoring_point_path_i);
      normalyse_node(isoring_point_path_i);
	

      if( path_grep(isoring_point_path_i,isorings_wk) == -1 ) {
	if(isorings_wk.count == 0 ) {
	  isorings_wk.count = 1;
	  isorings_wk.list = (int**)malloc(isorings_wk.count*sizeof(int*));
	}  else {
	  isorings_wk.count ++;
	  isorings_wk.list = (int**)realloc(isorings_wk.list,isorings_wk.count*sizeof(int*));
	}
	if( isorings_wk.list == 0 ) {
	  cout << "メモリ領域の確保に失敗しました : " << isorings_wk.count << endl;
	  exit(1);
	} else {
	  isorings_wk.list[isorings_wk.count - 1] = isoring_point_path_i;
	}
      } else {
	index_del(isoring_point_path_i);
      }

      if ( check_out_Edge( init_state , clique , nodenum , isoring_point_path_i ) == 0 ) {
	cout << "SSSR error(check_out_Edge == 3) " << endl;
	exit(1);
      }
    } 

    index_del(nodesN2);

    /*    
    cout << "fullSet[" << fullSet[0] << "]=";
    for( int i=1 ; i<= fullSet[0] ; i++ ) 
      cout << fullSet[i] << " ";
    cout << endl;
    cout << "trimSet[" << trimSet[0] << "]=";
    for( int i=1 ; i<= trimSet[0] ; i++ ) 
      cout << trimSet[i] << " ";
    cout << endl;
    */

  } while( ! compair_nodes(fullSet , trimSet) == 1 ) ;
  
  matrix_del(clique,nodenum);
  index_del(cliqueindex);

  return isorings_wk;

}

///////////////////////////////////////////////////////////////////////

int **search_neighbor_edge(int current, int **connection_pair, int edgenum) {
  // 隣接エッジを求める

  int **neighbor;
  neighbor = matrix_new(512,2);
  neighbor[0][0] = 0;
  neighbor[0][1] = 0;
  int s = connection_pair[current][0];
  int e = connection_pair[current][1];
  for( int i = 0 ; i < edgenum ; i ++ ) {
    if( i != current) {
      if( connection_pair[i][0] == s  || connection_pair[i][1] == s) {
	neighbor[0][0] ++;
	neighbor[ neighbor[0][0] ][0] = i;
	neighbor[ neighbor[0][0] ][1] = s;
      } else if( connection_pair[i][0] == e || connection_pair[i][1] == e ) {
	neighbor[0][0] ++;
	neighbor[ neighbor[0][0] ][0] = i;
	neighbor[ neighbor[0][0] ][1] = e;
      }
    }
  }
  return neighbor;
}

int check_member(int num, int *members) {
  // numがmembersに含まれているかどうかチェック。終端から捜す。
  if( members[0] != 0 ) {
    for( int i = members[0] ; i > 0 ; i -- ) {
      if( num == members[i] ) 
	return i;
    }
  }
  return 0;	   
}

int *add_member(int newmem, int *members, int max) {
  int *res;
  int dflg = 0;
  res = index_new(max+1);
  res[0] = members[0];

  for( int i = 1; i <= members[0]; i++ ) {
    res[i] = members[i];
    if( members[i] == newmem)
      dflg = 1;
  }
  if( dflg == 0 ) {
    res[0] ++;
    res[ res[0] ] = newmem;
  } else {
    //    cout << "member duplicated!!!!" << endl; 
  }
  return res;
}

pathlist connect_edge(int **connection_pair, int current_edge , int *root ,  pathlist isorings, int max_edge_num , int up_node , int edge_flg , int step){
  // 16 January 2009 T.Ohya
  // step はneighbor_edgeをいくつ先まで辿っているか

  pathlist isorings_wk;
  isorings_wk = isorings;
  int i;

//  cout << "max_edge=" <<  max_edge_num << endl;
//  cout << "current_edge=" <<  current_edge+1 << endl;
//  for( i = 0 ; i < max_edge_num  ; i ++ )
//  cout << i << "[" <<  connection_pair[i][0] << ":" << connection_pair[i][1] << "]" << endl;
//  cout << "---------" << endl;

  int **neighbor_edge;
  neighbor_edge = search_neighbor_edge( current_edge , connection_pair , max_edge_num);

  // neighbor_edge が 0なら孤立 1なら終端

  /*
  for( i = 1 ; i <= neighbor_edge[0][0] ; i ++ )
    cout << i << ":" << current_edge << ":" << neighbor_edge[i][0] << "(" <<  connection_pair[neighbor_edge[i][0]][0] << "-" << connection_pair[neighbor_edge[i][0]][1] << ")" << endl;
  */

  if( neighbor_edge[0][0] > 1 ) {

    int *root_wk;
    root_wk = add_member(current_edge,root,max_edge_num);

    int cnum = 0;

    for( i = 1 ; i <= neighbor_edge[0][0] ; i ++ ) {
      //     cout <<  "neighbor_edge = " << neighbor_edge[i][0] << endl;
      if( neighbor_edge[i][1] != up_node) {
	// 兄弟を対称nodeから除外
	int cnum_wk = check_member( neighbor_edge[i][0]  ,root_wk);
	if( cnum_wk > cnum ) 
	  cnum = cnum_wk;
	// cnumは数が大きい程 current_edgeに近い
      }
    }
    //    cout << "[" << root_wk[0] ;
    //    for( i = 1 ; i <= root_wk[0] ; i ++ ) {
    //      cout << ":" << root_wk[i] ;
    //    }
    //    cout << "]" << endl;

//  cout << "cnum=" << cnum << endl;
    if( cnum == 0 ) {
      // もし隣接エッジが 未知エッジなら、connectをくり返す。
      // step(connectのくり返し数)に限度を設ける　　16 January 2009 T.Ohya
      if (step < maximum_ringstep) {
        for( i = 1 ; i <= neighbor_edge[0][0] ; i ++ ) {
          if( neighbor_edge[i][1] != up_node) {
            // 兄弟を対称nodeから除外

//  cout << "<" << i+1 << ":" << connection_pair[neighbor_edge[i][0]][0]+1 << "-" <<  connection_pair[neighbor_edge[i][0]][1]+1 << ">" << endl;
            isorings_wk = connect_edge(connection_pair, neighbor_edge[i][0], root_wk, isorings_wk, max_edge_num , neighbor_edge[i][1] , edge_flg , step + 1);
          }
        }
      }
    } else {
      int *isoring_point_path_i;
      //      cout << "root = " << root_wk[0] + 1 << endl;
      isoring_point_path_i = index_new(root_wk[0] + 2);
      isoring_point_path_i[0] = 0;

//  cout << " ring (" ;
      
      for( i = cnum ; i <= root_wk[0] ; i ++ ) {
	int *current_pair;

	if( edge_flg == 0 ) {
	  // リング情報をノードで扱いたい場合はこっち
	  current_pair = index_new(3);
	  current_pair[0] = 2;
	  current_pair[1] = connection_pair[  root_wk[i] ][0];
	  current_pair[2] = connection_pair[  root_wk[i] ][1];
	
	} else {

	  // リング情報をエッジで扱いたい場合はこっち 
	  //	  cout << "(EDGE)" << endl;
	  current_pair = index_new(2);
	  current_pair[0] = 1;
	  current_pair[1] = root_wk[i];
	
	}
//cout <<  "[" << i << "/"  << connection_pair[root_wk[i]][0]+1 << "-" << connection_pair[root_wk[i]][1]+1 << "]" ;
	
	add_node( current_pair , isoring_point_path_i);
	index_del( current_pair );

      }
//cout << ")" << endl;

      sort_node(isoring_point_path_i);

      // outline情報を末尾に付加
      isoring_point_path_i[isoring_point_path_i[0] + 1 ] = isoring_point_path_i[0];

      /*
      cout << "Found Ring! : " << isoring_point_path_i[0] << ":" <<  isoring_point_path_i[isoring_point_path_i[0] + 1 ] << "(" ;

      cout << isoring_point_path_i[1]+1;
      for( int j = 2 ; j <= isoring_point_path_i[0] ; j ++ ) {
	cout << "-" << isoring_point_path_i[j]+1;
      }
      cout << ")" << endl;
      */

      if( path_grep(isoring_point_path_i,isorings_wk) == -1 ) {
	if(isorings_wk.count == 0 ) {
	  isorings_wk.count = 1;
	  isorings_wk.list = (int**)malloc(isorings_wk.count*sizeof(int*));

	}  else {
	  isorings_wk.count ++;
	  isorings_wk.list = (int**)realloc(isorings_wk.list,isorings_wk.count*sizeof(int*));

	}
	if( isorings_wk.list == 0 ) {
	  cout << "メモリ領域の確保に失敗しました : " << isorings_wk.count << endl;
	  exit(1);
	} else {
	  isorings_wk.list[isorings_wk.count - 1] = isoring_point_path_i;
	}
      } else {
      	index_del(isoring_point_path_i);
      }

      // もし隣接エッジが 既知エッジならリング
      // リング情報を追加し、探索を打ち切る。

    }
    index_del(root_wk);

    //  neibor_edge[0] == 0 
    // 隣接しているエッジが無い  
    //  neibor_edge[0] == 1 
    // 上流のエッジのみ存在 末端

  }
  matrix_del(neighbor_edge,10);
  return isorings_wk;

}

// isolated ringを探索する
// 深さ探索で計算する
// メモリエラー対策のため nodesの長さは固定する

pathlist search_isorings_depth(int **connection_pair, int nodenum, int edgenum, pathlist isorings , int edge_flg ) {

  pathlist isorings_wk;
  isorings_wk.count = 0;
  //  int i;

  for( int i = 0 ; i < edgenum ; i ++ ) { 

    int *root;
//    cout << "<" << i+1 << ":" << connection_pair[i][0]+1 << "-" <<  connection_pair[i][1]+1 << ">" << endl;
    root = index_new(edgenum + 1);
    root[0] = 1;
    root[1] = i;
    // エッジが分割されている場合を考慮して、全てのエッジについて計算する
    isorings_wk = connect_edge(connection_pair, i , root , isorings_wk, edgenum , -1 , edge_flg, 1);
    index_del(root);

  }
  return isorings_wk;
}

pathlist get_ring_status( pathlist rings ) {

  // rings同士の包含関係  隣接関係をチェックする。

  pathlist res; 
  res.count = rings.count;
  //  cout << " get_ring_status " << endl;
  res.list = (int**)malloc(rings.count*sizeof(int*));

  for( int i = 0 ; i < rings.count ; i ++) {
    int *path;
    path = index_new(res.count);
    res.list[i] = path;
  }

  for( int i = 0 ; i < rings.count ; i ++) {
    for( int j = i ; j < rings.count ; j ++) {
      int xor_flg = xor_path(rings.list[i] , rings.list[j] );
      if( xor_flg == 0 ) {
	// 同一リングである。
	res.list[i][j] = 0;
      } else if( xor_flg < 0 ) {
	// 不一致リング
	res.list[i][j] = -1;
	res.list[j][i] = -1;
      } else if( xor_flg == rings.list[j][0] ) {
	// j は i に 含まれる
	res.list[i][j] = 1;
	res.list[j][i] = 2;
      } else if( xor_flg == rings.list[i][0] ) {
	// i は j に 含まれる
	res.list[i][j] = 2;
	res.list[j][i] = 1;
      } else {
	// 隣接リングである。
	res.list[i][j] = 4;
	res.list[j][i] = 4;
      }
    }
  }
  return res;
}

/////////////////////////////////////////////////////

pathlist search_complexrings( pathlist isorings, pathlist complex_rings, int nodenum, int current_isoring_node ) {

  pathlist iso_status;
  iso_status =  get_ring_status( isorings );

  /*  
  for(int i = 0 ; i < iso_status.count ; i ++) {
    for(int j = 0 ; j < iso_status.count ; j ++) {
      cout << iso_status.list[i][j] << " ";
    }
    cout << endl;
  }
  */

  pathlist complex_rings_wk;
  int **cmp_status;

  complex_rings_wk = complex_rings;
  complex_rings_wk.count = 0;
  //  int outline1,outline2;

  int depth = 2;
  int dmax;
  int max_depth = 10;
 
  int start , end;
  start = 0;
  end = 0;

  // depth = 2 までの複合リングを計算する。

  //  cout << "   isoring x isoring " << endl;

  for(int i = 0 ; i < isorings.count ; i ++) {
    for(int j = i + 1 ; j < isorings.count ; j ++) {

      //      cout << i << ":" << j << endl;

      if( iso_status.list[i][j] == 4 ) {

	int *path_tmp;
	path_tmp = index_copy( isorings.list[i]  , nodenum + 1);
	add_node( isorings.list[j] , path_tmp);
	sort_node(path_tmp);

	//	cout << "a" << endl ;

	if( complex_rings_wk.count == 0 ) {
	  complex_rings_wk.count = 1;
	  complex_rings_wk.list = (int**)malloc(complex_rings_wk.count*sizeof(int*));
	  complex_rings_wk.list[0] = path_tmp;

	  cmp_status = (int**)malloc(complex_rings_wk.count*sizeof(int*));

	  int *cmp_tmp;
	  cmp_tmp = index_new( isorings.count + 1 );

	  for( int k = 0 ; k < isorings.count ; k ++ ) {
	    if( iso_status.list[i][k] == 4 ) {
	      cmp_tmp[k] = 1;
	    }
	    if( iso_status.list[j][k] == 4 ) {
	      cmp_tmp[k] = 1;
	    }
	  }

	  cmp_tmp[i] = -1; // subsetは-1
	  cmp_tmp[j] = -1;

	  cmp_status[0] = cmp_tmp; 

	} else {

	  //	  cout << " b" << endl;
	  int complex_rings_num = path_grep(path_tmp,complex_rings_wk);
	  if( complex_rings_num == -1 ) {	    
	    complex_rings_wk.count ++;
	    complex_rings_wk.list = (int**)realloc(complex_rings_wk.list,complex_rings_wk.count*sizeof(int*));
	    complex_rings_wk.list[complex_rings_wk.count - 1] = path_tmp;

	    cmp_status = (int**)realloc(cmp_status , complex_rings_wk.count*sizeof(int*));
	    //	    cout << "  c" << endl;
	    int *cmp_tmp;
	    cmp_tmp = index_new( isorings.count + 1 );

	    for( int k = 0 ; k < isorings.count ; k ++ ) {
	      if( iso_status.list[i][k] == 4 ) {
		cmp_tmp[k] = 1;
	      }
	      if( iso_status.list[j][k] == 4 ) {
		cmp_tmp[k] = 1;
	      }
	    }

	    //	    cout << "   d" << endl;
	    cmp_tmp[i] = -1; // subsetは-1
	    cmp_tmp[j] = -1;
	    cmp_status[ complex_rings_wk.count - 1 ] = cmp_tmp; 

	    //	    cout << "    e" << endl;
	  } else {
	    index_del(path_tmp);
	  }
	}
	//	cout << "*" << endl;
      }
      //      cout << "*" << endl;
    } // j
    end = complex_rings_wk.count - 1;
  } // i

  depth ++;

  if( max_depth < isorings.count ) 
    dmax = max_depth;
  else
    dmax = isorings.count;

  //  cout << "   cmpring x isoring " << endl;
  //  cout << "depth = " << depth << " : " << start << " to " << end << endl;
  //  cout << "L3 " << complex_rings_wk.count << endl;

  if( (int)complex_rings_wk.count > 0 ) {

    //    cout << "L3 " << complex_rings_wk.count << endl;
    while( depth <= dmax ) {
      // 3以上のリングについて、計算する。
      //      cout << "depth = " << depth << " : " << start << " to " << end << endl;
      
      for(int i = start ; i < end ; i ++) {
	//      cout << "  i = " << i << endl;
	for(int j = 0 ; j < isorings.count ; j ++) {
	  //	cout << "    j = " << j << " cmp_status = " << cmp_status[i][j] << endl;
	  
	  if( cmp_status[i][j] != -1 ) {
	    int *path_tmp;
	    path_tmp = index_copy( complex_rings_wk.list[i]  , nodenum + 1);
	    
	    //	  cout << "start" << endl;
	    if( xor_path( isorings.list[j] , path_tmp) > 1 ) {
	      
	      //	    cout << " oui" << endl;
	      add_node( isorings.list[j] , path_tmp);
	      sort_node(path_tmp);
	      
	      int complex_rings_num = path_grep(path_tmp,complex_rings_wk);
	      if( complex_rings_num == -1 ) {	    
		complex_rings_wk.count ++;
		complex_rings_wk.list = (int**)realloc(complex_rings_wk.list,complex_rings_wk.count*sizeof(int*));
		complex_rings_wk.list[complex_rings_wk.count - 1] = path_tmp;
		
		cmp_status = (int**)realloc(cmp_status , complex_rings_wk.count*sizeof(int*));
		int *cmp_tmp;
		cmp_tmp = index_new( isorings.count + 1 );
		
		//	      cout << "b" << endl;
		
		for( int k = 0 ; k < isorings.count ; k ++ ) {
		  if( isorings.list[j][k] == 4 ) {
		    cmp_tmp[k] = 1;
		  } else if( isorings.list[j][k] == -1 ) {
		    cmp_tmp[k] = -1;
		  }
		}
		cmp_tmp[j] = -1;
		cmp_status[ complex_rings_wk.count - 1 ] = cmp_tmp; 
		
	      } else {
		//     index_del(path_tmp);
	      }
	    } else { // xor_path
	      //	    cout << " non" << endl;
	      index_del(path_tmp);
	    }
	  }
	} // j
      } // i
      start = end + 1;
      
      end = complex_rings_wk.count - 1;
      depth ++;
      
    //    cout << depth << ":" << complex_rings_wk.count << ":" << maximum_cring << endl;

      if( complex_rings_wk.count > maximum_cring  ) {
	cout << "over flow of the number of the complex rings (" << complex_rings_wk.count << ")" << endl;
	depth = dmax + 1 ;
      }
    }
    free (cmp_status);
  } else {
    //    cout << "L4" << endl;
  }
  //  free (cmp_status);
  //  cout << "return" << endl;
  return complex_rings_wk;
}

// nodeで与えられた pathに対してoutlineを計算する。

void set_outline( pathlist rings  , int **cpair , int nodenum , int bondnum) {

  int count , s , e ;
  //  cout << "rings.count = " << rings.count << ":bondbum=" << bondnum << endl;;

  for(int i=0 ; i < rings.count ; i ++) {

    //    cout << i << ":" << endl; 

    count = 0;
    for(int j=0 ; j< bondnum ; j++ ) {

      s = 0;
      e = 0;

      //      cout << " j=" << j << endl;

      for( int k = 1 ; k <= rings.list[i][0]; k ++ ) {

	//	cout << "  k=" << k << endl;
	//	cout << "   rlist=" << rings.list[i][k] << endl;
	//	cout << "   cpair0=" << cpair[j][0] << endl;
	//	cout << "   cpair2=" << cpair[j][1] << endl;

	if( cpair[j][0] ==  rings.list[i][k] ) {
	  s = 1;
	} else if( cpair[j][1] ==  rings.list[i][k] ) {
	  e = 1;
	}
      }
      if( s * e == 1 ) 
	count ++;
    }
    //    cout << "()" << endl;
    rings.list[i][ rings.list[i][0] + 1 ] = count;
  }
  //  cout << endl;
}


//////////////////////////////////////////////////////

pathlist search_complexrings_edge( pathlist isorings, pathlist complex_rings, int edge_node , int current_isoring_node ) {

  // 全てのisoringを計算した筈なので、二つのリングを比較する
  // 本当は再帰的に全部やった方がいいんだけど、とりあえず暫定版

  pathlist complex_rings_wk;
  complex_rings_wk = complex_rings;

  //  cout << "isorings.count = " << isorings.count << endl;
  
  for(int i = 0 ; i < isorings.count ; i ++) {

    //    cout << i << endl;
    //    cout << "edge_node = " << edge_node << endl;

    int *path;
    path = index_copy(isorings.list[i] , edge_node + 1 );

    for(int j = i + 1 ; j < isorings.count ; j ++) {
      //       cout << " " << j << endl;

      int *path_tmp;
      
      path_tmp = index_copy(path, edge_node + 2);

      int outline1 = path_tmp[ path_tmp[0] + 1 ];
      int outline2 = isorings.list[j][ isorings.list[j][0] + 1 ];

      //      cout << " outline " << outline1 << ":" << outline2 << endl;

      // i番目のisoringsとのxorをとる。
      // 0 は自分自身 -1 は積無し 相手が自分のsubsetだった場合も0 
      // 1 は相手と1エッジで結合している場合なので、一致と判断する
      // isorings との比較なので0だった場合もcomplex_ringに属すると考える

      int xor_flg = xor_path(path_tmp,isorings.list[j] );
      if( xor_flg > 0 ) {
	add_node( isorings.list[j] , path_tmp);
	sort_node(path_tmp);
      }
      //      cout << " xor_flg = " << xor_flg << endl;

      if( xor_flg >= 0 ) {

	path_tmp[ path_tmp[0] + 1 ] = outline1 + outline2 - xor_flg;

	int complex_rings_num = path_grep(path_tmp,complex_rings_wk);
	if( complex_rings_num == -1 ) {	    
	  
	  if(complex_rings_wk.count == 0 ) {
	    complex_rings_wk.count = 1;
	    complex_rings_wk.list = (int**)malloc(complex_rings_wk.count*sizeof(int*));
	  }  else {
	    complex_rings_wk.count ++;
	    complex_rings_wk.list = (int**)realloc(complex_rings_wk.list,complex_rings_wk.count*sizeof(int*));
	  }

	  if( complex_rings_wk.list == 0 ) {
	    cout << "メモリ領域の確保に失敗しました : " << complex_rings_wk.count << endl;
	    exit(1);
	    
	  } else {

	    complex_rings_wk.list[complex_rings_wk.count - 1] = path_tmp;
	    //	    search_complexrings( isorings, complex_rings_wk, index_node, j , complex_rings_wk.count - 1  , depth_wk + 1);

	  } // メモリ領域の確保に成功しました

	} else {
	  // complex_ringsに属していた場合は追加しないので、データを解放
	  index_del(path_tmp);
	}
      }
    } // j
    index_del(path);
    //    cout << "del_path" << endl;
  } // i
  return complex_rings_wk;
}

